/*
 *  Msequence Encoder and Decoder
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include "msequence.h"

int main(argc, argv)
     int argc;
     char **argv;
{
  int i, j, n, op;
  int out;
  int length = (ChipWidth * SpreadingRate);
  short sig;
  unsigned char *code;
  double *data;

  /* options */
  int amplitude = 1000;
  int size = 5 * SF;
  int total = 0;
  char *output = "sample.wav";

  while((op = getopt(argc, argv, "a:o:s:")) != EOF){
    switch(op){
    case 'a':
      amplitude = atoi(optarg);
    case 'o':
      output = optarg;
      break;
    case 's':
      size = atoi(optarg) * SF;
      break;
    default:
      perror("invalid option");
      exit(1);
    }
  }
  argc -= optind;
  argv += optind;

  out = open(output, O_WRONLY | O_CREAT, 0644);
  if(out < 0){
    perror("output file open");
  }

  /* SS Parameter Initialization */ 
  code = (unsigned char *)code_generator();
  printf("[Msequence Bits]\n");
  for(i = 0; i < SpreadingRate; i++){
    printf("%d", code[i]);
  }
  printf("\n");
  
  data = modulation(code, ChipWidth);

  /* Wave Header Write */
  write_wave_header(out, size);  

  /* Encoding Loop */
  while(total < size){

    /* Signal Write */
    for(i = 0; i < length; i++){
      if(code[n]){
	sig = data[i] * amplitude;
      }else{
	sig = (-1) * data[i] * amplitude;
      }
      write(out, &sig, sizeof(short));
      total++;
    }

    if(total >= size){
      break;
    }
    
    n++;
    if(n == SpreadingRate){
      n = 0;
    }

  }
  
  printf("\n");

}


void write_wave_header(int fd, int size)
{

  struct wave_header *wh;
  wh = (struct wave_header *)malloc(sizeof(struct wave_header));

  strncpy(wh->chunkID, "RIFF", 4);
  wh->chunksize = (size * 2) + 36;
  strncpy(wh->format, "WAVE", 4);
  strncpy(wh->fmtID, "fmt ", 4);
  wh->fmtsize = 16;
  wh->waveformat = 1;
  wh->channel = 1;
  wh->sample_rate = SF;
  wh->bps = SF * 2;
  wh->blocksize = 2;
  wh->bits = 16;
  strncpy(wh->dataID, "data", 4);
  wh->datasize = size * 2;
  
  write(fd, (char *)wh, 44);

}







